package com.podpoint.pptmapview;

import android.content.Context;
import com.google.android.gms.maps.MapView;
import com.google.android.gms.maps.OnMapReadyCallback;
import com.google.android.gms.maps.GoogleMap;
import com.facebook.react.bridge.LifecycleEventListener;
import com.facebook.react.bridge.ReactContext;
import com.facebook.react.bridge.ReactApplicationContext;
import com.facebook.react.uimanager.ThemedReactContext;

public class PPTGoogleMapView extends MapView implements OnMapReadyCallback
{
    /**
     * React lifecycle listener
     */
    private LifecycleEventListener lifecycleListener;

    /**
     * React context
     */
    private ThemedReactContext reactContext;

    /**
     * The google map instance
     */
    public GoogleMap googleMap;

    /**
     * Is the map view paused
     */
    private boolean paused = false;

    /**
     * Is the map view destroyed
     */
    private boolean destroyed = false;

    /**
     * Mapview contructor
     *
     * @param reactContext
     */
    public PPTGoogleMapView(ThemedReactContext reactContext) {
        super(reactContext);

        this.reactContext = reactContext;
    }

    /**
     * Event handler for when map is ready to receive update parameters.
     *
     * @param googleMap
     */
    @Override
    public void onMapReady(final GoogleMap googleMap) {
        if (destroyed) {
            return;
        }

        this.googleMap = googleMap;

        // Set LifecycleEventListener to set MapView on pause/resume/destroy, to avoid killing battery
        lifecycleListener = new LifecycleEventListener() {
            @Override
            public void onHostResume() {
                googleMap.setMyLocationEnabled(true);
                synchronized (PPTGoogleMapView.this) {
                    onResume();
                    paused = false;
                }
            }

            @Override
            public void onHostPause() {
                googleMap.setMyLocationEnabled(false);
                synchronized (PPTGoogleMapView.this) {
                    onPause();
                    paused = true;
                }
            }

            @Override
            public void onHostDestroy() {
                PPTGoogleMapView.this.doDestroy();
            }
        };

        reactContext.addLifecycleEventListener(lifecycleListener);
    }

    /**
     * Handle map view destroy
     */
    public synchronized void doDestroy() {
        if (destroyed) {
            return;
        }
        destroyed = true;

        if (lifecycleListener != null && reactContext != null) {
            reactContext.removeLifecycleEventListener(lifecycleListener);
            lifecycleListener = null;
        }
        if (!paused) {
            onPause();
            paused = true;
        }
        onDestroy();
    }
}

